@@ -2,26 +2,22 @@
 
 import numpy as np
 from predictit import misc
-from IPython import get_ipython
 
 
-def compare_predicted_to_test(predicted, test, train=None, error_criterion='mape', plot=0, modelname="Default model", dataname="default data", details=0):
+def compare_predicted_to_test(predicted, test, error_criterion='mape', plot=0, modelname="Default model", dataname="default data"):
     """Compare tested model with reality.
 
     Args:
         predicted (np.ndarray): Model output.
         test (np.ndarray): Correct values or output from data_pre funcs.
-        train (np.ndarray, optional): Real history values for plotting - for plot olny!. Defaults to None.
         error_criterion (str, optional): 'mape' or 'rmse'. Defaults to 'mape'.
         plot (int, optional): Whether create plot. Defaults to 0.
         modelname (str, optional): Model name for plot. Defaults to "Default model".
         dataname (str, optional): Data name for plot. Defaults to "default data".
-        details (int, optional): Whether print details. Defaults to 0.
 
     Returns:
         float: Error criterion value (mape or rmse).
         If in setup - return  plot of results.
-
     """
 
     predicts = len(predicted)
@@ -32,7 +28,9 @@ def compare_predicted_to_test(predicted, test, train=None, error_criterion='mape
 
     if predicted is not None:
         if plot:
+
             if misc._JUPYTER:
+                from IPython import get_ipython
                 get_ipython().run_line_magic('matplotlib', 'inline')
 
             import matplotlib.pyplot as plt
@@ -46,18 +44,6 @@ def compare_predicted_to_test(predicted, test, train=None, error_criterion='mape
             plt.title("Prediction with \n {} with data {}".format(modelname, dataname))
             plt.show()
 
-            if train is not None:  # TODO delete if work also for date
-                tt = range(len(predicted) * 10)
-                window = len(predicted) * 9
-                predictedpluslast = np.insert(predicted, 0, train[-1])
-
-                plt.plot(tt[:window], train[-window:])
-                plt.plot(tt[window - 1:], predictedpluslast)
-                plt.xlabel('t')
-                plt.ylabel("Predicted value")
-                plt.title("History + predictions with \n {} with data {}".format(modelname, dataname))
-                plt.show()
-
         error = np.array(predicted) - np.array(test)
 
         '''
@@ -87,7 +73,4 @@ def compare_predicted_to_test(predicted, test, train=None, error_criterion='mape
         else:
             raise KeyError(misc.colorize(f"bad 'error_criterion' config - '{error_criterion}'. Use some from options from config comment..."))
 
-        if details == 1:
-            print(f"Error of model {modelname} on data {dataname}: {error_criterion} = {criterion_value}")
-
         return criterion_value
