@@ -3,9 +3,9 @@ import sklearn.ensemble
 import sklearn.multioutput
 import sklearn.tree
 import sklearn.neighbors
-from sklearn import linear_model
 import numpy as np
-import sklearn_extensions.extreme_learning_machines.elm as elm
+
+linear_model = sklearn.linear_model
 
 
 def get_regressors():
@@ -25,7 +25,8 @@ def get_regressors():
     return regressors
 
 
-def train(sequentions, regressor='bayesianridge', predicts=7, n_estimators=100,
+def train(sequentions, regressor='bayesianridge', predicts=7, load_trained_model=0, update_trained_model=1, save_model=1,
+          saved_model_path_string='stored_models', n_estimators=100,
           alpha=0.0001, alpha_1=1.e-6, alpha_2=1.e-6, lambda_1=1.e-6,
           lambda_2=1.e-6, n_iter=300, epsilon=1.35, alphas=[0.1, 0.5, 1], gcv_mode='auto', solver='auto',
           n_hidden=20, rbf_width=0, activation_func='selu', **kwargs):
@@ -53,51 +54,69 @@ def train(sequentions, regressor='bayesianridge', predicts=7, n_estimators=100,
     Returns:
         np.ndarray: Predictions of input time series.
     """
+
     if regressor == 'bayesianridge':
-        regressor_model = linear_model.BayesianRidge(n_iter=n_iter, alpha_1=alpha_1, alpha_2=alpha_2, lambda_1=lambda_1, lambda_2=lambda_2)
+        model = linear_model.BayesianRidge(n_iter=n_iter, alpha_1=alpha_1, alpha_2=alpha_2, lambda_1=lambda_1, lambda_2=lambda_2)
     elif regressor == 'huber':
-        regressor_model = linear_model.HuberRegressor(epsilon=epsilon, max_iter=200, alpha=alpha)
+        model = linear_model.HuberRegressor(epsilon=epsilon, max_iter=200, alpha=alpha)
     elif regressor == 'lasso':
-        regressor_model = linear_model.Lasso(alpha=alpha)
+        model = linear_model.Lasso(alpha=alpha)
     elif regressor == 'linear':
-        regressor_model = linear_model.LinearRegression()
+        model = linear_model.LinearRegression()
     elif regressor == 'ridgecv':
-        regressor_model = linear_model.RidgeCV(alphas=alphas, gcv_mode=gcv_mode)
+        model = linear_model.RidgeCV(alphas=alphas, gcv_mode=gcv_mode)
     elif regressor == 'ridge':
-        regressor_model = linear_model.Ridge(alpha=alpha, solver=solver)
+        model = linear_model.Ridge(alpha=alpha, solver=solver)
     elif regressor == 'Extra trees':
-        regressor_model = sklearn.ensemble.ExtraTreesRegressor(n_estimators=n_estimators)
+        model = sklearn.ensemble.ExtraTreesRegressor(n_estimators=n_estimators)
     elif regressor == 'Random forest':
-        regressor_model = sklearn.ensemble.RandomForestRegressor()
+        model = sklearn.ensemble.RandomForestRegressor()
     elif regressor == 'Decision tree':
-        regressor_model = sklearn.tree.DecisionTreeRegressor()
+        model = sklearn.tree.DecisionTreeRegressor()
     elif regressor == 'Gradient boosting':
-        regressor_model = sklearn.ensemble.GradientBoostingRegressor()
+        model = sklearn.ensemble.GradientBoostingRegressor()
     elif regressor == 'KNeighbors':
-        regressor_model = sklearn.neighbors.KNeighborsRegressor()
+        model = sklearn.neighbors.KNeighborsRegressor()
     elif regressor == 'Bagging':
-        regressor_model = sklearn.ensemble.BaggingRegressor()
+        model = sklearn.ensemble.BaggingRegressor()
     elif regressor == 'Stochastic gradient':
-        regressor_model = sklearn.linear_model.SGDRegressor()
+        model = sklearn.linear_model.SGDRegressor()
     elif regressor == 'Passive aggressive regression':
-        regressor_model = sklearn.linear_model.PassiveAggressiveRegressor()
+        model = sklearn.linear_model.PassiveAggressiveRegressor()
     elif regressor == 'elm':
-        regressor_model = elm.ELMRegressor(n_hidden=n_hidden, alpha=alpha, rbf_width=rbf_width, activation_func=activation_func)
+        import sklearn_extensions.extreme_learning_machines.elm as elm
+        model = elm.ELMRegressor(n_hidden=n_hidden, alpha=alpha, rbf_width=rbf_width, activation_func=activation_func)
     elif regressor == 'elm_gen':
-        regressor_model = elm.GenELMRegressor()
+        import sklearn_extensions.extreme_learning_machines.elm as elm
+        model = elm.GenELMRegressor()
 
-    model= sklearn.multioutput.MultiOutputRegressor(regressor_model)
+    if sequentions[1].shape[1] == 1:
+        model.output_shape = 'one_step'
+        output = sequentions[1].ravel()
 
-    model.fit(sequentions[0], sequentions[1])
+    else:
+        model = sklearn.multioutput.MultiOutputRegressor(model)
+        model.output_shape = 'batch'
+        output = sequentions[1]
 
-    model.output_shape = sequentions[1].shape[1]
+    model.fit(sequentions[0], output)
 
     return model
 
 
 def predict(x_input, model, predicts=7):
+    """Function that creates predictions from trained model and input data.
 
-    if model.output_shape == 1:
+    Args:
+        data (np.ndarray): Time series data
+        model (list, class): Trained model. It can be list of neural weigths or it can be fitted model class from imported library.
+        predicts (int, optional): Number of predicted values. Defaults to 7.
+
+    Returns:
+        np.ndarray: Array of predicted results
+    """
+
+    if model.output_shape == 'one_step':
 
         predictions = []
 
